ELBO calculating functions depending on type of model for covariance matrix
Source:R/ELBO_function_type.R
elbo_fixed_diagonal.RdELBO calculating functions depending on type of model for covariance matrix
R/ELBO_function_type.R
elbo_fixed_diagonal.RdELBO calculating functions depending on type of model for covariance matrix